EvNameMainPlayerAppeared: str = 'MainPlayerAppeared'
EvNamePlayerAppeared: str = 'PlayerAppeared'
EvNameDamageDone: str = 'DamageDone'
EvNameHealthReceived: str = 'HealthReceived'
EvNameZoneChange: str = 'ZoneChange'
EvNameEnterCombat: str = 'EnterCombat'
EvNameLeaveCombat: str = 'LeaveCombat'
EvNameUpdateFame: str = 'UpdateFame'
EvNameUpdateItems: str = 'UpdateItems'
EvNameUpdateParty: str = 'UpdateParty'

EvKeyName: str = 'name'
EvKeyValue: str = 'value'
EvKeySource: str = 'source'
EvKeyTarget: str = 'target'
EvKeyId: str = 'id'
EvKeyPlayerNames: str = 'player_names'
